@@ -2,12 +2,22 @@
 
 #' @title Map protein-relative coordinates to positions within the transcript
 #'
+#' @name proteinToTranscript
+#'
+#' @aliases proteinToTranscript proteinToTranscript,EnsDb-method
+#'
 #' @description
 #'
 #' `proteinToTranscript` maps protein-relative coordinates to positions within
 #' the encoding transcript. Note that the returned positions are relative to
 #' the complete transcript length, which includes the 5' UTR.
 #'
+#' The regions within the protein sequence need to be provided as a named
+#' `IRanges` object with the names being protein identifiers and the start and
+#' end coordinates (within these proteins) defined by the `IRanges` object. As
+#' an alternative to the `IRanges`' names, protein identifiers can also be
+#' provided through a metadata column (see details below).
+#'
 #' Similar to the [proteinToGenome()] function, `proteinToTranscript` compares
 #' for each protein whether the length of its sequence matches the length of
 #' the encoding CDS and throws a warning if that is not the case. Incomplete
@@ -35,6 +45,8 @@
 #'
 #' @inheritParams proteinToGenome
 #'
+#' @param ... Further arguments to be passed on.
+#'
 #' @return
 #'
 #' `IRangesList`, each element being the mapping results for one of the input
@@ -100,8 +112,11 @@
 #'
 #' ## The result for the region within the second protein
 #' res[[2]]
-proteinToTranscript <- function(x, db, id = "name",
-                                idType = "protein_id") {
+setGeneric("proteinToTranscript", signature="db",
+    function(x, db, ...) standardGeneric("proteinToTranscript")
+)
+setMethod("proteinToTranscript", "EnsDb",
+  function(x, db, id = "name", idType = "protein_id") {
     if (missing(x) || !is(x, "IRanges"))
         stop("Argument 'x' is required and has to be an 'IRanges' object")
     if (missing(db) || !is(db, "EnsDb"))
@@ -149,6 +164,8 @@ proteinToTranscript <- function(x, db, id = "name",
                 else mc$protein_id <- names(prt)
                 ir <- IRanges(start = -1, width = 1)
                 mcols(ir) <- mc
+                if(!is.null(mcols(prt)))
+                    mcols(ir) <- cbind(mcols(ir),mcols(prt))
                 ir
             } else {
                 ids <- names(gnm)
@@ -167,11 +184,132 @@ proteinToTranscript <- function(x, db, id = "name",
                 if (idType == "uniprot_id")
                     mc$uniprot_id <- names(prt)
                 mcols(res) <- mc
+                if(!is.null(mcols(prt)))
+                    mcols(res) <- cbind(mcols(res),mcols(prt))
                 res
             }
         }), "IRangesList")
-}
+})
 
+#' @rdname proteinToTranscript
+#'
+#' @aliases proteinToTranscript proteinToTranscript,Preloaded-method
+#'
+#' @inheritParams proteinToGenome
+#'
+#' @param fiveUTR A `CompressedGRangesList` object generated by
+#'     [fiveUTRsByTranscript()].
+#'
+#' @family coordinate mapping functions
+#'
+#' @author Johannes Rainer
+#'
+#' @md
+#'
+#' @examples
+#'
+#' ## Meanwhile, this function can be called in parallel processes if you preload
+#' ## the CDS data with desired data columns and fiveUTR data
+#'
+#' cds <- cdsBy(edbx,columns = c(listColumns(edbx,'tx'),'protein_id','uniprot_id','protein_sequence'))
+#' # cds <- cdsBy(edbx,columns = c(listColumns(edbx,'tx'),'protein_id','protein_sequence'))
+#' # cds <- cdsBy(edbx,columns = c('tx_id','protein_id','protein_sequence'))
+#'
+#' fiveUTR <- fiveUTRsByTranscript(edbx)
+#'
+#' ## Define an IRange with protein-relative coordinates within a protein for
+#' ## the gene SYP
+#' syp <- IRanges(start = 4, end = 17)
+#' names(syp) <- "ENSP00000418169"
+#' res <- proteinToTranscript(syp, cds, fiveUTR = fiveUTR)
+#' res
+#' ## Positions 4 to 17 within the protein span are encoded by the region
+#' ## from nt 23 to 64.
+#'
+#' ## Perform the mapping for multiple proteins identified by their Uniprot
+#' ## IDs.
+#' ids <- c("O15266", "Q9HBJ8", "unexistant")
+#' prngs <- IRanges(start = c(13, 43, 100), end = c(21, 80, 100))
+#' names(prngs) <- ids
+#'
+#' res <- proteinToTranscript(prngs, cds, idType = "uniprot_id", fiveUTR = fiveUTR)
+setMethod("proteinToTranscript", "CompressedGRangesList",
+  function(x, db, id = "name", idType = "protein_id", fiveUTR) {
+    if (missing(x) || !is(x, "IRanges"))
+        stop("Argument 'x' is required and has to be an 'IRanges' object")
+    if (missing(db) || !is(db, "CompressedGRangesList"))
+        stop("Argument 'db' is required and has to be an 'CompressedGRangesList' object")
+    if (missing(fiveUTR) || !is(fiveUTR, "CompressedGRangesList"))
+        stop("Argument 'fiveUTR' is required and has to be an 'CompressedGRangesList' object")
+    coords_cds <- .proteinCoordsToTx(x)
+    ## 1) retrieve CDS for each protein
+    message("Fetching CDS for ", length(x), " proteins ... ",
+            appendLF = FALSE)
+    cds_genome <- .cds_for_id_range(db, x, id = id, idType = idType)
+    miss <- lengths(cds_genome) == 0
+    if (any(miss))
+        warning("No CDS found for: ", paste0(names(cds_genome)[miss],
+                                             collapse = ", "))
+    message(sum(!miss), " found")
+    ## 2) ensure that the CDS matches the AA sequence length
+    message("Checking CDS and protein sequence lengths ... ", appendLF = FALSE)
+    cds_genome <- .cds_matching_protein(cds_genome)
+    are_ok <- vapply(cds_genome, function(z) {
+        if (is(z, "GRangesList"))
+            all(z[[1]]$cds_ok)
+        else NA
+    }, FUN.VALUE = logical(1))
+    are_ok <- are_ok[!is.na(are_ok)]
+    ## We've got now a list of GRanges
+    message(sum(are_ok), "/", length(are_ok), " OK")
+    ## Get for each transcript it's 5' UTR and add its width to the coords_cds
+    tx_ids <- unique(unlist(lapply(cds_genome, names), use.names = FALSE))
+    if (length(tx_ids)) {
+        five_utr <- fiveUTR[names(fiveUTR) %in% tx_ids]
+        ## Calculate 5' widths for these
+        five_width <- sum(width(five_utr))
+    }
+    as(mapply(
+        cds_genome, as(coords_cds, "IRangesList"), split(x, 1:length(x)),
+        FUN = function(gnm, cds, prt) {
+            if (is.null(gnm)) {
+                ## Define the metadata columns
+                mc <- DataFrame(protein_id = NA_character_,
+                                tx_id = NA_character_,
+                                cds_ok = NA,
+                                protein_start = start(prt),
+                                protein_end = end(prt))
+                if (idType == "uniprot_id")
+                    mc$uniprot_id <- names(prt)
+                else mc$protein_id <- names(prt)
+                ir <- IRanges(start = -1, width = 1)
+                mcols(ir) <- mc
+                if(!is.null(mcols(prt)))
+                    mcols(ir) <- cbind(mcols(ir),mcols(prt))
+                ir
+            } else {
+                ids <- names(gnm)
+                res <- IRanges(start = start(cds) + five_width[ids],
+                               end = end(cds) + five_width[ids],
+                               names = ids)
+                ## Populate mcols
+                mc <- DataFrame(
+                    protein_id = unlist(
+                        lapply(gnm, function(z) z$protein_id[1])),
+                    tx_id = ids,
+                    cds_ok = gnm[[1]]$cds_ok[1],
+                    protein_start = start(prt),
+                    protein_end = end(prt)
+                )
+                if (idType == "uniprot_id")
+                    mc$uniprot_id <- names(prt)
+                mcols(res) <- mc
+                if(!is.null(mcols(prt)))
+                    mcols(res) <- cbind(mcols(res),mcols(prt))
+                res
+            }
+        }), "IRangesList")
+})
 
 #' @title Map within-protein coordinates to genomic coordinates
 #'
@@ -187,9 +325,16 @@ proteinToTranscript <- function(x, db, id = "name",
 #' (and eventually Uniprot to Ensembl protein identifier mappings) from the
 #' submitted `EnsDb` object (and thus based on annotations from Ensembl).
 #'
-#' Not all coding regions for protein coding transcripts are complete, and the
-#' function thus checks also if the length of the coding region matches the
-#' length of the protein sequence and throws a warning if that is not the case.
+#' The regions within the protein sequence need to be provided as a named
+#' `IRanges` object with the names being protein identifiers and the start and
+#' end coordinates (within these proteins) defined by the `IRanges` object. As
+#' an alternative to the `IRanges`' names, protein identifiers can also be
+#' provided through a metadata column (see details below).
+#'
+#' Note that not all coding regions for protein coding transcripts are
+#' complete, and the function thus checks also if the length of the coding
+#' region matches the length of the protein sequence and throws a warning if
+#' that is not the case.
 #'
 #' The genomic coordinates for the within-protein coordinates, the Ensembl
 #' protein ID, the ID of the encoding transcript and the within protein start
@@ -232,7 +377,7 @@ proteinToTranscript <- function(x, db, id = "name",
 #'     encoding transcripts.
 #'
 #' @param id `character(1)` specifying where the protein identifier can be
-#'     found. Has to be either `"name"` or one of `colnames(mcols(prng))`.
+#'     found. Has to be either `"name"` or one of `colnames(mcols(x))`.
 #'
 #' @param idType `character(1)` defining what type of IDs are provided. Has to
 #'     be one of `"protein_id"` (default), `"uniprot_id"` or `"tx_id"`.
@@ -341,8 +486,10 @@ setMethod("proteinToGenome", "EnsDb",
     are_ok <- are_ok[!is.na(are_ok)]
     ## We've got now a list of GRanges
     ## Perform the mapping for each input range with each mapped cds
+    x_IRangesList <- as(x, "IRangesList") ## preserve the metadata
+    x_IRangesList@unlistData@elementMetadata <- x@elementMetadata
     res <- mapply(
-        cds_genome, as(coords_cds, "IRangesList"), as(x, "IRangesList"),
+        cds_genome, as(coords_cds, "IRangesList"), x_IRangesList,
         FUN = function(gnm, cds, prt) {
             if (!length(gnm)) { ## addresses NULL and empty elements
                 GRanges()
@@ -355,6 +502,101 @@ setMethod("proteinToGenome", "EnsDb",
                 mcols(maps) <- cbind(mcols(maps),
                                      protein_start = start(prt),
                                      protein_end = end(prt))
+                if(!is.null(mcols(prt)))
+                    mcols(maps) <- cbind(mcols(maps),mcols(prt))
+                maps[order(maps$exon_rank)]
+            }
+        })
+    ## Split each element again, if there are more than one protein_id. Names
+    ## of the elements are then the protein_id.
+    lapply(res, function(z) {
+        if (length(unique(z$protein_id)) > 1)
+            split(z, f = z$protein_id)
+        else z
+    })
+  })
+
+#' @rdname proteinToGenome
+#'
+#' @aliases proteinToGenome proteinToGenome,Preloaded-method
+#'
+#' @param db For the method for `EnsDb` objects: An `EnsDb` object to be used to
+#'     retrieve genomic coordinates of encoding transcripts.<br><br>
+#'     For the method for `CompressedGRangesList` objects: A `CompressedGRangesList` object
+#'     generated by [cdsBy()] where by = 'tx' and columns = c('tx_id'
+#'     ,'protein_id','uniprot_id','protein_sequence').
+#'
+#' @md
+#'
+#' @examples
+#'
+#' ## Meanwhile, this function can be called in parallel processes if you preload
+#' ## the CDS data with desired data columns
+#' cds <- cdsBy(edbx,columns = c(listColumns(edbx,'tx'),'protein_id','uniprot_id','protein_sequence'))
+#' # cds <- cdsBy(edbx,columns = c(listColumns(edbx,'tx'),'protein_id','protein_sequence'))
+#' # cds <- cdsBy(edbx,columns = c('tx_id','protein_id','protein_sequence'))
+#' ## Define an IRange with protein-relative coordinates within a protein for
+#' ## the gene SYP
+#' syp <- IRanges(start = 4, end = 17)
+#' names(syp) <- "ENSP00000418169"
+#' res <- proteinToGenome(syp, cds)
+#' res
+#' ## Positions 4 to 17 within the protein span two exons of the encoding
+#' ## transcript.
+#'
+#' ## Perform the mapping for multiple proteins identified by their Uniprot
+#' ## IDs.
+#' ids <- c("O15266", "Q9HBJ8", "unexistant")
+#' prngs <- IRanges(start = c(13, 43, 100), end = c(21, 80, 100))
+#' names(prngs) <- ids
+#'
+#' res <- proteinToGenome(prngs, cds, idType = "uniprot_id")
+setMethod("proteinToGenome", "CompressedGRangesList",
+  function(x, db, id = "name", idType = "protein_id") {
+    if (missing(x) || !is(x, "IRanges"))
+        stop("Argument 'x' is required and has to be an 'IRanges' object")
+    if (missing(db) || !is(db, "CompressedGRangesList"))
+        stop("Argument 'db' is required and has to be an 'CompressedGRangesList' object")
+    coords_cds <- .proteinCoordsToTx(x)
+    ## 1) retrieve CDS for each protein
+    message("Fetching CDS for ", length(x), " proteins ... ",
+            appendLF = FALSE)
+    cds_genome <- .cds_for_id_range(db, x, id = id, idType = idType)
+    miss <- lengths(cds_genome) == 0
+    if (any(miss))
+        warning("No CDS found for: ", paste0(names(cds_genome)[miss],
+                                             collapse = ", "))
+    message(sum(!miss), " found")
+    ## 2) ensure that the CDS matches the AA sequence length
+    message("Checking CDS and protein sequence lengths ... ", appendLF = FALSE)
+    cds_genome <- .cds_matching_protein(cds_genome)
+    are_ok <- unlist(lapply(cds_genome, function(z) {
+        if (is(z, "GRangesList") & length(z) > 0)
+            all(z[[1]]$cds_ok)
+        else NA
+    }))
+    message(sum(are_ok, na.rm = TRUE), "/", length(are_ok), " OK")
+    are_ok <- are_ok[!is.na(are_ok)]
+    ## We've got now a list of GRanges
+    ## Perform the mapping for each input range with each mapped cds
+    x_IRangesList <- as(x, "IRangesList") ## preserve the metadata
+    x_IRangesList@unlistData@elementMetadata <- x@elementMetadata
+    res <- mapply(
+        cds_genome, as(coords_cds, "IRangesList"), x_IRangesList,
+        FUN = function(gnm, cds, prt) {
+            if (!length(gnm)) { ## addresses NULL and empty elements
+                GRanges()
+            } else {
+                ## Unlist because we'd like to have a GRanges here. Will split
+                ## again later.
+                maps <- unlist(.to_genome(gnm, cds))
+                ## Don't want to have GRanges names!
+                names(maps) <- NULL
+                mcols(maps) <- cbind(mcols(maps),
+                                     protein_start = start(prt),
+                                     protein_end = end(prt))
+                if(!is.null(mcols(prt)))
+                    mcols(maps) <- cbind(mcols(maps),mcols(prt))
                 maps[order(maps$exon_rank)]
             }
         })
@@ -437,7 +679,7 @@ setMethod("proteinToGenome", "EnsDb",
 #'
 #' Use one query to fetch CDS for all (unique) input IDs. If input IDs are
 #' Uniprot identifiers we have to perform additional checks and data
-#' re-organizations because one transript (and thus CDS) can be associated
+#' re-organizations because one transcript (and thus CDS) can be associated
 #' with multiple Uniprot identifiers.
 #'
 #' @param x `EnsDb` object.
@@ -464,7 +706,7 @@ setMethod("proteinToGenome", "EnsDb",
         tx_id <- split(map$tx_id, map[, idType])
     } else {
         tx_id <- id
-        split(tx_id, id)
+        tx_id <- split(tx_id, id)
     }
     if (length(tx_id)) {
         suppressWarnings(
@@ -490,6 +732,83 @@ setMethod("proteinToGenome", "EnsDb",
     cds
 }
 
+#' @description Fetch the CDS for all transcripts encoding the specified protein using
+#'     preloaded data.
+#'
+#' @note
+#'
+#' Use one query to fetch CDS for all (unique) input IDs using preloaded data. If input
+#' IDs are Uniprot identifiers we have to perform additional checks and data
+#' re-organizations because one transcript (and thus CDS) can be associated
+#' with multiple Uniprot identifiers.
+#'
+#' @param x `CompressedGRangesList` object generated by [cdsBy()] where
+#'     by = 'tx' and columns = c(listColumns(edb,'tx'),'protein_id','uniprot_id','protein_sequence').
+#'
+#' @param id `character` with the protein ID(s).
+#'
+#' @param idType `character(1)` defining what type of IDs are provided. Has to
+#'     be one of `"protein_id"` (default), `"uniprot_id"` or `"tx_id"`.
+#'
+#' @return a `list` with the CDS of the encoding transcript(s) for each provided
+#'     id (as a `GRangesList`). Names of the `list` are the ids, if no
+#'     transcript was found `NULL` is returned.
+#'
+#' @author Johannes Rainer, Boyu Yu
+#'
+#' @md
+#'
+#' @noRd
+setMethod(".cds_for_id2", "CompressedGRangesList",
+  function(x, id, idType = "protein_id") {
+    ## No need for get the transcripts first, just use the metadata of CDS
+    ## if you find this modification well, you may want to change the
+    ## generic .cds_for_id2 function above
+    if (!all(c("tx_id", "protein_id",idType,"protein_sequence") %in% colnames(mcols(x[[1]]))))
+        stop(
+            paste(
+                c(
+                    "CDS must have",
+                    unique(c("'tx_id'","'protein_sequence'", "'protein_id'",paste("'",idType,"'",sep=''))),
+                    "in the metadata."
+                ),
+            collapse = ' ')
+        )
+    map <- x@unlistData[as.data.frame(x@partitioning)[,1]]@elementMetadata[,unique(c('tx_id', 'protein_id', idType))]
+    map <- map[map[,idType] %in% id,]
+    if (idType != "tx_id") {
+        tx_id <- split(map$tx_id, map[, idType])
+    } else {
+        tx_id <- id
+        tx_id <- split(tx_id, id)
+    }
+    if (length(tx_id)) {
+        cds <- x[names(x) %in% unique(unlist(tx_id, use.names = FALSE))]
+        if (length(cds)) {
+            if (idType == "uniprot_id") {
+                ## Additional (in)sanity checks for Uniprot identifiers
+                ## This has a significant impact on performance, but otherwise
+                ## we end up with duplicated transcript entries!
+                tmp <- unlist(cds)
+                tmp <- as.list(split(unname(tmp), tmp$uniprot_id))
+                cds <- lapply(tmp, function(z) split(z, z$tx_id))
+            } else if ("uniprot_id" %in% colnames(mcols(x[[1]]))) {
+                warning(
+                    "Multiple uniprot ids were preserved for the same transcript entry. ",
+                    "Please don't include uniprot_id column in cdsBy() if unneeded."
+                )
+                cds <- lapply(tx_id, function(z) cds[z])
+            } else {
+                cds <- lapply(tx_id, function(z) cds[z])
+            }
+        }
+        else cds <- list()
+    } else cds <- list()
+    cds <- cds[id]
+    names(cds) <- id # to add also names for elements not found
+    cds
+})
+
 #' @description
 #'
 #' Uses .cds_for_id to fetch CDS for protein identifiers and in addition
@@ -514,10 +833,17 @@ setMethod("proteinToGenome", "EnsDb",
 #' @noRd
 .cds_for_id_range <- function(x, range, id = "name", idType = "protein_id") {
     idType <- match.arg(idType, c("protein_id", "uniprot_id", "tx_id"))
-    id <- match.arg(id, c("name", colnames(mcols(range))))
+    if (!(id %in% c("name", colnames(mcols(range)))))
+        stop("'id' should be either \"name\" (if identifiers are provided as ",
+             "the IRanges' names) or one of the column names of the IRanges ",
+             "metadata (mcols()).", call. = FALSE)
     if (id == "name")
         ids <- names(range)
     else ids <- mcols(range)[, id]
+    if (length(ids) != length(range) | all(is.na(ids)))
+        stop("No protein/transcript identifiers found. Please provide the ",
+             "IDs through the IRanges' names or one of its metadata columns.",
+             call. = FALSE)
     cds <- .cds_for_id2(x, ids, idType = idType)
     ## check returned CDS if their width matches the provided protein ranges
     mapply(cds, end(range) * 3,
@@ -546,6 +872,8 @@ setMethod("proteinToGenome", "EnsDb",
 #'
 #' @param cds `GRangesList`
 #'
+#' @param ... Further arguments to be passed on..
+#'
 #' @return `list` of `GRangesList`s with one list element per input protein,
 #'    and the `GRangesList` containing the CDS of all matching (or one not
 #'    matching) transcripts.
@@ -555,7 +883,11 @@ setMethod("proteinToGenome", "EnsDb",
 #' @md
 #'
 #' @noRd
-.cds_matching_protein <- function(x, cds) {
+setGeneric(".cds_matching_protein", signature="x",
+    function(x, ...) standardGeneric(".cds_matching_protein")
+)
+setMethod(".cds_matching_protein", "EnsDb",
+  function(x, cds) {
     ## Fetch the protein sequences, all in one go.
     ## Loop through the cds
     prot_ids <- unique(unlist(lapply(cds, function(z) {
@@ -596,7 +928,67 @@ setMethod("proteinToGenome", "EnsDb",
             }
         }
     })
-}
+})
+
+#' @description
+#'
+#' Fetches the protein sequence using the provided `"protein_id"` in `cds` and
+#' checks whether the cds lenghts match the protein sequence. The
+#' function returns all CDS for each protein/transcript that have the correct
+#' length) and throws a warning if none of the specified CDS matches the
+#' protein sequence. In the latter case a single CDS (the one with the CDS
+#' length closest to the expected length). Whether a CDS has the expected length
+#' or not is also reported in the metadata column `"cds_ok"`.
+#'
+#' @details
+#'
+#' Mismatch between the CDS and the AA sequence are in most instances caused by
+#' incomplete (3' or 5') CDS sequences.
+#'
+#' @param x `list` object generated by .cds_for_id2.
+#'
+#'
+#' @return `list` of `GRangesList`s with one list element per input protein,
+#'    and the `GRangesList` containing the CDS of all matching (or one not
+#'    matching) transcripts.
+#'
+#' @author Johannes Rainer, Boyu Yu
+#'
+#' @md
+#'
+#' @noRd
+setMethod(".cds_matching_protein", "list",
+  function(x) {
+    mapply(x, names(x), FUN = function(z, nm) {
+        if (!is.null(z)) {
+            cds_lens <- sum(width(z))
+            exp_cds_len <- unlist(lapply(z, function(y)  nchar(unique(y$protein_sequence)) * 3 + 3))
+            ## Calculate the expected CDS length (add +3 to add the stop codon).
+            diffs <- cds_lens - exp_cds_len
+            ## Return all for which diff is 0, or otherwise the one with the
+            ## smallest diff.
+            if (any(diffs == 0)) {
+                z <- lapply(z, function(grng) {
+                    mcols(grng)$cds_ok <- TRUE
+                    grng
+                })
+                GRangesList(z[diffs == 0])
+            } else {
+                warning("Could not find a CDS whith the expected length for ",
+                        "protein: '", nm, "'. The returned genomic ",
+                        "coordinates might thus not be correct for this ",
+                        "protein.", call. = FALSE)
+                ## Alternatively we could align the RNA and AA sequences and
+                ## trim the protein sequence...
+                z <- lapply(z, function(grng) {
+                    mcols(grng)$cds_ok <- FALSE
+                    grng
+                })
+                GRangesList(z[which.min(abs(diffs))])
+            }
+        }
+    })
+})
 
 #' @description
 #'
